
// auxiliares.h

void SetRAMBank() {
#asm
	.SetRAMBank
	ld	A, ($5B5C)
	and F8h
	or	B
	ld	BC, $7FFD
	ld	($5B5C), A
	out (C), A
#endasm
}

/* MusicISR */
#asm
defw 0	// 2 bytes libres
#endasm

void ISR(void)
{
	if (is128k) {
		#asm
			ld b,1
			call SetRAMBank
			call WYZPLAYERISR
			ld b,0
			call SetRAMBank
		#endasm
	}
}

void unpack_RAMX (unsigned char ram, unsigned int address) {
	asm_int_2 [0] = address;
	asm_char [0] = ram;
	
	#asm	
		di
		ld a, (_asm_char)
		ld b, a
		call SetRAMBank
		
		ld hl, (_asm_int_2)
		ld de, 16384
		call depack
		
		ld b, 0
		call SetRAMBank
		ei
	#endasm
}

void unpack_loRAM (unsigned int address) {
	asm_int_2 [0] = address;
	
	#asm
		ld hl, (_asm_int_2)
		ld de, 16384
		call depack
	#endasm 
}

void *my_malloc(unsigned int bytes) {
   return sp_BlockAlloc(0);
}

void *u_malloc = my_malloc;

void *u_free = sp_FreeBlock;

void wyz_play_sound (unsigned char fx_number, unsigned char fx_channel) {
	if (!is128k)
		return;
	
	asm_int_2 [0] = fx_channel + (fx_number * 256);
	
	#asm
		di
		ld b,1
		call SetRAMBank
		ld bc, (_asm_int_2)
		call INICIAEFECTO
		ld b,0
		call SetRAMBank
		ei
	#endasm
}

void wyz_play_music (unsigned char song_number) {
	if (!is128k)
		return;
	
	asm_int_2 [0] = song_number;

	#asm
		di
		ld b, 1
		call SetRAMBank
		ld a, (_asm_int_2)
		call CARGA_CANCION
		ld b, 0
		call SetRAMBank
		ei
	#endasm
}

void wyz_stop_sound ()
{
	if (!is128k)
		return;
	
	#asm
		di
		ld b,1
		call SetRAMBank
		call SILENCIA_PLAYER
		ld b,0
		call SetRAMBank
		ei
	#endasm
}

void fade_out () { 
	/* El segundo programa de ms de 5 lineas en Z80 ASM de na_th_an */ 
	/* Optimizado por Metalbrain */ 
	/* Esto "oscurece" un paso la pantalla. Llamado 8 veces la dejar en negro */ 
	unsigned char i;
	
	for (i = 0; i < 8; i ++) {	
		#asm 
			ld	 e, 3				; 3 tercios 
			ld	 hl, 22528			; aqu empiezan los atributos 
			halt					; esperamos retrazo. 
		.fade_out_bucle  
			ld	 a, (hl )			; nos traemos el atributo actual 
			
			ld	 d, a				; tomar atributo 
			and  7					; aislar la tinta 
			jr	 z, ink_done		; si vale 0, no se decrementa 
			dec  a					; decrementamos tinta 
		.ink_done 
			ld	 b, a				; en b tenemos ahora la tinta ya procesada. 
			
			ld	 a, d				; tomar atributo 
			and  56 				; aislar el papel, sin modificar su posición en el byte 
			jr	 z, paper_done		; si vale 0, no se decrementa 
			sub  8					; decrementamos papel restando 8 
		.paper_done 
			ld	 c, a				; en c tenemos ahora el papel ya procesado. 
			ld	 a, d  
			and  192				; nos quedamos con bits 6 y 7 (BRIGHT y FLASH) 
			or	 c					; aadimos paper 
			or	 b					; e ink, con lo que recompuesto el atributo 
			ld	 (hl),a 			; lo escribimos, 
			inc  l					; e incrementamos el puntero. 
			jr	 nz, fade_out_bucle ; continuamos hasta acabar el tercio (cuando L valga 0) 
			inc  h					; siguiente tercio 
			dec  e 
			jr	 nz, fade_out_bucle ; repetir las 3 veces 
		#endasm 
	}
}

unsigned char rand () {
	unsigned char res;
	
	#asm
	
	; modo brutito: ON
	
			;ld 	a,r
	._Rand8 ld		a,9 	; Seed is usually 0
			ld		b,a
			add 	a,a
			add 	a,a
			add 	a,b
			;inc	a		; another possibility is ADD A,7
			add 	a, 7
			ld (_Rand8+1),a
			ld (_asm_number), a
	#endasm

	res = asm_number [0];

	return res;
}

void blackout_everything () {
	unsigned char *origin;

	origin = (unsigned char *) 22528;
	while (origin < 23296) {
		*origin = 0;
		origin ++;
	}
}

char espera_activa (unsigned int espera) {
	// Esta funcin espera un rato o hasta que se pulse una tecla.
	// Si se pulsa una tecla, devuelve 0
	
	// Esta funcin slo funciona en Spectrum.
	// en CPC no hay una interrupcin cada 20ms, asn que esto no
	// sirve "pa n".
	
	char res = 1;
	unsigned int i;
	
	for (i = 0; i < espera && res; i ++) {
		#asm
			halt
		#endasm
		if (sp_GetKey ()) 
			res = 0;
	}
	
	return res;
}

void halt_me (int espera) {
	int i;
	
	for (i = 0; i < espera; i ++) {
		#asm
			halt
		#endasm
	}
}

void my_PrintString (char *string) {
	// He hecho esta funcin para no tener que tener (:D) los carcteres
	// en sus ASCII correspondientes. Uno, qus asn.
	unsigned char index = 0;
	unsigned char x = 0, y = 0, atrib = 7;
	unsigned char fin = 0;
	
	while (!fin) {
		switch (string [index]) {
			case 0x10:					// ATR
				index ++;
				atrib = string [index];
				break;
			case 0x16:					// AT
				index ++;
				y = string [index];
				index ++;
				x = string [index];
				break;
			case 0xff:
				fin = 1;
				break;
			default:
				sp_PrintAtInv (y, x, atrib, string [index] - 32);
				x = x + 1;
		}
		index ++;
	}
}

void wait_for_fire () {
	unsigned char i;
	
	do {
			i = (joyfunc) (&keys);
	} while ( (i & sp_FIRE) != 0 ); 
}

void scroll_pal_final () {
	#asm
		; Prueba de una ventana escrolleante parriba. Con que lo haga un PICSEL, 
		; lo tengo hecho...
		
		; la ventana es de cuatro chars de alto por 14 de ancho y est en y=2,x=18.
		
		; Este cdigo es una MIERDA pero ahora mismo no doy para ms.
		; Ya ir aprendiendo, coo.
		
					halt
		
					ld		hl, 16466				; Aqu empieza (2, 18)
					ld		de, 16466
					inc 	d						; de apunta a la fila de abajo
		
					ld		(dinicial), hl
					
					ld		c, 4					; Recorrer cuatro lineas de chars
		.jcrb1		push	bc
		
					ld		(remember), hl
		
					ld		c, 7					; Recorrer 8 filas de pxels
		.jcrb2		push	bc
		
					ld		c, 14					; Recorrer 13 carcteres
		.jcrb3		
		
					ld		a, (de) 				; Copiamos
					ld		(hl), a
					
					inc 	l
					inc 	e						; Siguiente byte
		
					
					dec 	c
					jr		nz, jcrb3
					
					ld		a, l					; Nos movemos al principio de la siguiente linea.
					sub 	14
					ld		l, a
					ld		e, a
					inc 	h
					inc 	d
		
					pop 	bc
					dec 	c
					jr		nz, jcrb2
					
					; Copiar la ltima linea de la siguiente de la siguiente 
					ld		c, 14
					; Ahora HL est al principio de la ltima linea
					; Y queremos copiar la siguiente, pero esto es
					; lo complicao... BURRR
					ld		de, (remember)
					ld		a, e	
					add 	a, 32
					ld		e, a
					
		.jcrb4		
					ld		a, (de)
					ld		(hl), a
		
					inc l
					inc e
		
					dec 	c
					jr		nz, jcrb4
					
					
					ld		hl, (dinicial)			; Recalculamos el inicio para la siguiente linea
					ld		a, l
					add 	a, 32
					ld		l, a
					
					ld		(dinicial), hl
					ld		de, (dinicial)
					inc 	d
					
					pop 	bc
					dec 	c
					jr		nz, jcrb1
					
					ret
					
		.dinicial	defw	0
		.remember	defw	0
	#endasm 
}
