/*

  ######### ###    ### ###    ###   ######## ###     ###
  ###     # ###   #### ###    ###  ###   ###  ###   ###
  ###       ###  ##### ###    ### ###    ###   ### ###
  #######   ### ###### ##########  ###   ###    #####
  ###  ###  ###### ### ###    ###   ########   ### ###
  ###   ### #####  ### ###    ###  ###   ###  ###   ###
  ###  ###  ####   ### ###    ### ###    ### ###     ###
  #######   ###    ### ###    ### ###    ### 
  
  v.2.0 C para z88dk / splib2 - Sinclair ZX Spectrum 128
  
*/

#pragma output STACKPTR=25000

/* Plan de vuelo
   =============
   
   1 [ ] Hacer el motor sencillo del juego en una funcin que reciba
         los parmetros necesarios de la fase. Las vidas y los puntos
         son variables globales.
         
   2 [ ] Montar a pelo las pginas y hacer la lista de llamadas a la
         funcin juego con los parmetros adecuados de cada fase.
         
*/

// Includes:

#include <spritepack.h>
#include "externs.h"			// Mira: esto permite definir aqu externs y poner los datos en plan guarro al final
								// Esto mola sobre todo porque as paginar es mucho ms fcil (e divertente).
#include "aplib.h"

// Defines:

#define AD_FREE         60000   // 58000

#define TABLERO_X 		12
#define TABLERO_Y 		1

#define MARCADOR_BLK_X	15
#define MARCADOR_BLK_Y	23
#define MARCADOR_TS_X	30
#define MARCADOR_TS_Y	23

#define INICIAEFECTO	0xC364
#define WYZPLAYERISR	0xC021	// Esto es INICIO en el cdigo de wyzplayer.asm
#define CARGA_CANCION	0xC05E
#define SILENCIA_PLAYER	0xC043

#define STATE_IDLE		0
#define STATE_WLEFT 	1
#define STATE_WRIGHT	2
#define STATE_WUP		3
#define STATE_WDOWN 	4

#define CANAL_EFECTOS	1

// Direcciones de memoria de pantallas estticas en RAM 6

#define S_MOJON			49152
#define S_UBHRES		49513
#define S_MENU			50716
#define S_GOVER			53382
#define S_FIN			54412

// Struct/types:

typedef struct {
	unsigned int puntos;
	unsigned char vidas;
} DATOS_PLAYER;

typedef struct {
	unsigned char c1;
	unsigned char c2;
} FICHA;

typedef struct {
	unsigned char x;
	unsigned char y;
	unsigned char p;
	unsigned char mct;
	unsigned char ct;
	unsigned int mn_fichas;
	unsigned int n_fichas;
	unsigned char prendas;
	char xstp;
	char ystp;
	unsigned char estado;
} NANAKO;

// Globales

extern struct sp_Rect *sp_ClipStruct;
extern char asm_number [1];
extern unsigned char asm_char [1];
extern unsigned int asm_int_2 [1];
extern char *Rand8;
unsigned char *allpurposepuntero;
struct sp_Rect spritesClipValues;
struct sp_Rect *spritesClip;
struct sp_UDK keys;
void *joyfunc;				// Puntero a la funcin de manejo seleccionada.
DATOS_PLAYER player;
unsigned char is128k;
FICHA tablero [35];			// Tablero de 5x7
NANAKO nanako;
struct sp_Rect rectangulo;

// Strings
unsigned char ss_level [] = "\x10\x4e\x16\x09\x0a           \x16\x0a\x0a LEVEL  -  \x16\x0b\x0a           \x16\x0c\x0a GET READY \x16\x0d\x0a           \xff";
unsigned char ss_princess_of [] = "\x10\x47\x16\x0a\x13PRINCESS FROM\xff";
unsigned char ss_loses_a_cloth [] = "\x10\x05\x16\x0f\x13LOSES A CLOTH\xff";
unsigned char ss_final [] = "FINALLY.      YOU DEFEATED  ALL THE NEARBYKINGDOM'S     PRINCESSES,,, NOW NANAKO    WILL BE TAKEN SERIOUSLY,,,  SEE YOU       IN SKYTHORIA!                                                         ";
unsigned char ss_menu [] = "\x10\x47\x16\x0A\x021, KEYBOARD\x16\x0B\x022, KEMPSTON\x16\x0D\x023, START GAME\x16\x0E\x024, NEVERENDING\xff";
unsigned char ss_cleared [] = "\x10\x46\x16\x0F\x0CCLEARED!\xff";

// Extern data:

#asm
    LIB SPCClipStruct
._sp_ClipStruct
    defw SPCClipStruct
._asm_number 
    defb 0
._asm_int_2
	defw 0
._asm_char
	defb 0
#endasm

/***************************
 FUNCIONES AUXILIARES SYSTEM
 ***************************/

#include "auxiliares.h"

/******************
 FUNCIONES DE JUEGO
 ******************/

#include "logo.h" 
#include "game.h"

/*********
 PRINCIPAL
 *********/
 
void main (void) {
	char aux;
	int aux_tiles;
	char level;
	char gametype = 0;
	int key_1, key_2, key_3, key_4;
	char control = 0;
	
	// Sustituye ISR
	
    #asm
        di
    #endasm

    sp_InitIM2(0xf1f1);
    sp_CreateGenericISR(0xf1f1);
    sp_RegisterHook(255, ISR);

    #asm
        ei
    #endasm

    #asm
        ; Chequea si son 128k y lo pone en la variable is128k
        ; Code by BlackHole
        LD      HL,$F1FF
        LD      (HL),1
        LD      A,$01
        LD      BC,7FFDh
        OUT     (C),A
        LD      (HL),0
        LD      A,($5B5C)
        AND     $F8
        LD      BC,$7FFD
        OUT     (C),A
        LD      A,(HL)
        LD      (_is128k),A
    #endasm
    
    // Inicializa splib2

    sp_Initialize (0, 0);
    sp_Border (BLACK);
    sp_AddMemory(0, 50, 14, AD_FREE);       // Allocate for 50 blocks of memory (tight!)
    
    //ps.flags  = sp_PSS_INVALIDATE;
    //ps.bounds = sp_ClipStruct;

    // Clip structure

    spritesClipValues.row_coord = 0;
    spritesClipValues.col_coord = 0;
    spritesClipValues.height = 24;
    spritesClipValues.width = 32;
    spritesClip = &spritesClipValues;
    
    // Definimos las teclas
    
    keys.up    = sp_LookupKey ('q');
    keys.down  = sp_LookupKey ('a');
    keys.left  = sp_LookupKey ('o');
    keys.right = sp_LookupKey ('p');
    keys.fire  = sp_LookupKey (' ');
    
    key_1 = sp_LookupKey ('1');
    key_2 = sp_LookupKey ('2');
    key_3 = sp_LookupKey ('3');
    key_4 = sp_LookupKey ('4');
        
    // Definir tileset
    
    allpurposepuntero = main_tileset;
    for (aux_tiles = 0; aux_tiles < 256; aux_tiles++)
    {
        sp_TileArray ((unsigned char)aux_tiles, allpurposepuntero);
        allpurposepuntero += 8;
    }
    
    // Empezamos!
    
    joyfunc = sp_JoyKeyboard;
    control = 0;
    
    unpack_RAMX (6, S_MOJON);
    wyz_play_sound (1, CANAL_EFECTOS);
    if (espera_activa (120)) {
	    fade_out ();
    	unpack_RAMX (6, S_UBHRES);
    	wyz_play_sound (1, CANAL_EFECTOS);
    	espera_activa (120);
	}
    fade_out ();
    
    while (1) {
	    sp_ClearRect(sp_ClipStruct, 0, 0, sp_CR_TILES);
		sp_UpdateNow ();
		blackout_everything ();
		unpack_RAMX (6, S_MENU);
		
		wyz_play_music (2);
		
		my_PrintString (ss_menu);
		
		gametype = 0;
		
		while (!gametype) {
			sp_PrintAtInv (10 + control, 14, 70, '<'-32);
			sp_PrintAtInv (11 - control, 14, 70, ' '-32);
			sp_UpdateNow ();
			
			if (sp_KeyPressed (key_1)) {
				control = 0;
				joyfunc = sp_JoyKeyboard;
			}
				
			if (sp_KeyPressed (key_2)) {
				joyfunc = sp_JoyKempston;		
				control = 1;
			}
				
			if (sp_KeyPressed (key_3))
				gametype = 1;
				
			if (sp_KeyPressed (key_4))
				gametype = 2;
		}
		
		wyz_stop_sound ();
		fade_out ();
		
		if (gametype == 1) {
		    level = 0;
		    do {
		    	aux = play_world (level);
		    	level ++;
			} while (!aux && level < 4);
			
			if (aux) {
				// Game over
				wyz_play_music (3);
				unpack_RAMX (6, S_GOVER);	
				espera_activa (250);
				wyz_stop_sound ();
			} else {
				// Final
				
				final ();						
			}
		} else {
			nanako.prendas = 3;
			aux = do_game (6, 999, 10);
			
			if (aux) {
				// Game over
				
				unpack_RAMX (6, S_GOVER);	
				espera_activa (250);
			}	
		}
	}
}

#include "tiles.h"
#include "marcadores.h"

/*	COSAS DE DISEO
	
	A ver, para que sea menos cdigo, voy a codificar todas las fases de esta manera en una
	tabla de estructuras:
	
	Tabla mundos:
	
	0	n_princesa		byte*10		Nombre de la princesa.
	10	n_land_1		byte*10		Nombre del sitio, linea 1.
	20	n_land_2		byte*10		Nombre del sitio, linea 2.
	30	l1				byte		ndice en la tabla levels de la primera fase; -1 = no hay fase.
	31	l2				byte
	32	l3				byte
	33	l4				byte
	34	l5				byte
	35	add_scr			byte*2		Pantalla de presentacin del mundo
	(37 bytes)
	
	Tabla fases:
	
	0	n_colores		byte		N mximo de colores.
	1	n_fichas		byte		N mximo de fichas.
	2	n_counter		byte		N mximo de contador de scroll.
	3	scr_ram			byte		En qu pgina de RAM est el dibujo.
	4	scr_addr		byte*2		Direccin del dibujo.
	(6 bytes)
*/

#include "fases.h"
	